package com.wgmf.library.wx_share;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Log;
import android.widget.Toast;

import com.tencent.mm.sdk.modelmsg.SendMessageToWX;
import com.tencent.mm.sdk.modelmsg.WXImageObject;
import com.tencent.mm.sdk.modelmsg.WXMediaMessage;
import com.tencent.mm.sdk.modelmsg.WXTextObject;
import com.tencent.mm.sdk.modelmsg.WXWebpageObject;
import com.tencent.mm.sdk.openapi.IWXAPI;
import com.tencent.mm.sdk.openapi.WXAPIFactory;
import com.wgmf.library.utils.BaseSingleton;

/**
 * 微信分享管理类
 *
 * @author xiep
 */
public class WechatShareManager {

    private static final int THUMB_SIZE = 150;
    /**
     * 文字
     */
    private static final int WEIXIN_SHARE_WAY_TEXT = 1;
    /**
     * 图片
     */
    private static final int WEIXIN_SHARE_WAY_PIC = 2;
    /**
     * 链接
     */
    private static final int WEIXIN_SHARE_WAY_WEBPAGE = 3;
    /**
     * 会话
     */
    public static final int WEIXIN_SHARE_TYPE_TALK = SendMessageToWX.Req.WXSceneSession;

    private IWXAPI wxApi;
    private Context context;

    private static BaseSingleton<WechatShareManager, Void> instance = new BaseSingleton<WechatShareManager, Void>() {
        @Override
        protected WechatShareManager create(Void var1) {
            return new WechatShareManager();
        }
    };

    public static WechatShareManager getInstance() {
        return instance.get(null);
    }

    private WechatShareManager() {

    }

    public void initWechatShareManager(Context context) {
        this.context = context;
        //初始化数据
        String wechatAppId = WechatShareUtil.getWeixinAppId();
        //初始化微信分享代码
        if (wechatAppId != null) {
            wxApi = WXAPIFactory.createWXAPI(context, wechatAppId, true);
            wxApi.registerApp(wechatAppId);
        }
    }

    /**
     * 通过微信分享
     *
     * @param shareContent 分享的方式（文本、图片、链接）
     * @param shareType    分享的类型（朋友圈，会话）
     */
    public void shareByWeixin(ShareContent shareContent, int shareType) {
        if (!wxApi.isWXAppInstalled()) {
            Toast.makeText(context, "您还未安装微信客户端",
                    Toast.LENGTH_SHORT).show();
            return;
        }
        switch (shareContent.getShareWay()) {
            case WEIXIN_SHARE_WAY_TEXT:
                shareText(shareType, shareContent);
                break;
            case WEIXIN_SHARE_WAY_PIC:
                sharePicture(shareType, shareContent);
                break;
            case WEIXIN_SHARE_WAY_WEBPAGE:
                shareWebPage(shareType, shareContent);
                break;
            default:
                break;
        }
    }

    private abstract class ShareContent {
        protected abstract int getShareWay();

        protected abstract String getContent();

        protected abstract String getTitle();

        protected abstract String getURL();

        protected abstract int getPicResource();

    }

    /**
     * 设置分享链接的内容
     *
     * @author Administrator
     */
    public class ShareContentWebpage extends ShareContent {
        private String title;
        private String content;
        private String url;
        private int picResource;

        public ShareContentWebpage(String title, String content,
                                   String url, int picResource) {
            this.title = title;
            this.content = content;
            this.url = url;
            this.picResource = picResource;
        }

        @Override
        protected String getContent() {
            return content;
        }

        @Override
        protected String getTitle() {
            return title;
        }

        @Override
        protected String getURL() {
            return url;
        }

        @Override
        protected int getPicResource() {
            return picResource;
        }

        @Override
        protected int getShareWay() {
            return WEIXIN_SHARE_WAY_WEBPAGE;
        }

    }

    /**
     * 分享文字
     */
    private void shareText(int shareType, ShareContent shareContent) {
        String text = shareContent.getContent();
        //初始化一个WXTextObject对象
        WXTextObject textObj = new WXTextObject();
        textObj.text = text;
        //用WXTextObject对象初始化一个WXMediaMessage对象
        WXMediaMessage msg = new WXMediaMessage();
        msg.mediaObject = textObj;
        msg.description = text;
        //构造一个Req
        SendMessageToWX.Req req = new SendMessageToWX.Req();
        //transaction字段用于唯一标识一个请求
        req.transaction = buildTransaction("textshare");
        req.message = msg;
        //发送的目标场景， 可以选择发送到会话 WXSceneSession 或者朋友圈 WXSceneTimeline。 默认发送到会话。
        req.scene = shareType;
        wxApi.sendReq(req);
    }

    /**
     * 分享图片
     */
    private void sharePicture(int shareType, ShareContent shareContent) {
        Bitmap bmp = BitmapFactory.decodeResource(context.getResources(), shareContent.getPicResource());
        WXImageObject imgObj = new WXImageObject(bmp);

        WXMediaMessage msg = new WXMediaMessage();
        msg.mediaObject = imgObj;

        Bitmap thumbBmp = Bitmap.createScaledBitmap(bmp, THUMB_SIZE, THUMB_SIZE, true);
        bmp.recycle();
        //设置缩略图
        msg.thumbData = WechatShareUtil.bmpToByteArray(thumbBmp);

        SendMessageToWX.Req req = new SendMessageToWX.Req();
        req.transaction = buildTransaction("imgshareappdata");
        req.message = msg;
        req.scene = shareType;
        wxApi.sendReq(req);
    }

    /**
     * 分享链接
     */
    private void shareWebPage(int shareType, ShareContent shareContent) {
        try {
            WXWebpageObject webpage = new WXWebpageObject();
            webpage.webpageUrl = shareContent.getURL();
            WXMediaMessage msg = new WXMediaMessage(webpage);
            msg.title = shareContent.getTitle();
            msg.description = shareContent.getContent();

            Bitmap thumb = BitmapFactory.decodeResource(context.getResources(), shareContent.getPicResource());
            if (thumb == null) {
                Toast.makeText(context, "图片不能为空", Toast.LENGTH_SHORT).show();
            } else {
                msg.thumbData = WechatShareUtil.bmpToByteArray(thumb);
                if (!thumb.isRecycled()) {
                    thumb.recycle();
                }
            }
            SendMessageToWX.Req req = new SendMessageToWX.Req();
            req.transaction = buildTransaction("webpage");
            req.message = msg;
            req.scene = shareType;
            wxApi.sendReq(req);
        } catch (Exception e) {
            Log.e("WechatShareManager", "Error shareWebPage:", e);
        }
    }

    private String buildTransaction(final String type) {
        return (type == null) ? String.valueOf(System.currentTimeMillis()) : type + System.currentTimeMillis();
    }
}